/*
 * 
 */

package	com.idca.fsim.core.indexer;

import	com.idca.fsim.core.FsimContext;
import	com.idca.fsim.core.plugins.SearchSource;
import java.io.IOException;

/**
 * An request to index
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 22, 2006
 */
public class IndexRequest
{
	/** */
	private	SearchSource searchSource = null;
	/** */
	private FsimContext fsimContext = null;
	/** The Stats of an IndexRequest */
	public enum RequestState { init,send,running,finished };
	/** The current state of this IndexRequest */
	private RequestState requestState = null;
	
	/**
	 * Creates an IndexRequest for an SearchSource
	 * @param searchSource
	 * @param fsimContext
	 */
	public IndexRequest(SearchSource searchSource,FsimContext fsimContext)
	{
		if(searchSource==null) {
			throw new NullPointerException("searchSource may not be null.");
		}
		if(fsimContext==null) {
			throw new NullPointerException("fsimContext may not be null.");
		}
		this.searchSource=searchSource;
		this.fsimContext=fsimContext;
		this.requestState=RequestState.init;
	}
	
	/**
	 * Executes this IndexRequest
	 *
	 */
	public void execute() {
		Indexer i = new Indexer(this);
		IndexerManagerListener.getIndexerManager().getThreadPoolExecutor().execute(i);
		requestState = RequestState.send;
	}
	
	/**
	 * Returns the current state of the request.
	 * @return	Returns true when the worker is finished.
	 */
	protected void running() {
		requestState = RequestState.running;
	}
	
	/**
	 * This methodes should be call by the worker to indicate that he is ready.
	 * Also this methode closed and flushed all the streams.
	 * 
	 * And we notify all(one!!) the threads waiting. 
	 * @throws IOException	When een IO error is found.
	 */
	protected void finished() throws IOException {
		requestState = RequestState.finished;
	}
	
	/**
	 * Gets the SearchSource
	 * @return Returns the SearchSource
	 */
	public SearchSource getSearchSource() {
		return searchSource;
	}
	
	/**
	 * Gets the FsimContext.
	 * @return Returns the FsimContext
	 */
	public FsimContext getFsimContext() {
		return fsimContext;
	}
	
	/**
	 * Gets the current request state.
	 * @return Returns the state.
	 */
	public RequestState getRequestState() {
		return requestState;
	}
	
}
