/*
 * 
 * 
 */

package	com.idca.fsim.core.indexer;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * An indexer
 * 
 * @author Willem Cazander
 * @version 1.0 20/07/2005
 */
public class Indexer implements Runnable
{
	
	/** The Logger to log to. */
	private Logger logger = null;
	/** The request we most handle. */
	private IndexRequest indexRequest = null;

	public Indexer(IndexRequest indexRequest) {
		logger = Logger.getLogger(this.getClass().getName());	
		if(indexRequest==null) {
			throw new IllegalArgumentException("indexRequest may not be null.");
		}
		this.indexRequest=indexRequest;
	}
	
	public void run() {
		
		try {
			long startTime = System.currentTimeMillis();
			String requestName = indexRequest.getSearchSource().getName();
			logger.info("Taking request to index : "+requestName);
			indexRequest.running();
			// do indexing
			indexRequest.getSearchSource().getIndexer().index(indexRequest.getSearchSource());
			
			long endTime = System.currentTimeMillis();
			logger.info("Finished indexing: "+requestName+" was in in "+(endTime-startTime)+" ms.");
			
			/*
			IObjectProfileNode profile = ObjectProfiler.profile(indexRequest.getFsimContext());
			int size = profile.size();
			System.out.println("fsimcontext size: "+size);
			
			profile = ObjectProfiler.profile(indexRequest.getFsimContext().getWordLists());
			System.out.println("wordslists size: "+profile.size());
			System.out.println("(context-wordslists): "+(size-profile.size()));
			*/
			
		} catch (Exception e) {
			logger.log(Level.WARNING,"Error while running indexer.",e);
		} finally {
			try {
				indexRequest.finished();
			} catch (IOException ioe) {
				logger.log(Level.WARNING,"Error while setting request to finished.",ioe);
			}
		}
	}
	
}
