
package com.idca.fsim.core.jdbc;

import java.sql.Connection;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;	
import java.util.logging.Logger;

/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 4, 2006
 *
 * Manages the different ConnectionProviders.
 * 
 */
public class ConnectionProviderManager
{
	/** The logger to log to */
	private Logger logger = null;
	/** The map with all connection providers */
	private Map<String,ConnectionProvider> connectionProviders = null;
	
	/**
	 * Creates an ConnectionProviderManager
	 *
	 */
	public ConnectionProviderManager() {
		logger = Logger.getLogger(ConnectionProviderManager.class.getName());
		logger.fine("Creating an ConnectionProviderManager");
		connectionProviders = new HashMap<String,ConnectionProvider>(3);
	}
	
	
	/**
	 * Gets an Connection by its connectionName
	 * @param connectionName	The connection name.
	 * @return	Returns the Connection
	 */
	public Connection getConnection(String connectionName) {
		return getConnectionProvider(connectionName).getConnection();
	}
	
	/**
	 * Adds an ConnectionProvider
	 * @param connectionProvider The ConnectionProvider to add.
	 */
	public void addConnectionProvider(ConnectionProvider connectionProvider) {
		if(connectionProvider==null) {
			throw new NullPointerException("connectionProvider may not be null.");
		}
		logger.fine("Adding ConnnectionProvider for connection: "+connectionProvider.getConnectionName());
		connectionProviders.put(connectionProvider.getConnectionName(),connectionProvider);
	}
	
	/**
	 * Gets an ConnectionProvider by its connectionName
	 * @param connectionName	The connection name
	 * @return	Returns the ConnectionProvider for this connectionName.
	 */
	public ConnectionProvider getConnectionProvider(String connectionName) {
		return connectionProviders.get(connectionName);
	}
	
	/**
	 * Gets all ConnectionProviders
	 * @return Returns all ConnectionProviders.
	 */
	public Collection<ConnectionProvider> getConnectionProviders() {
		return connectionProviders.values();
	}
}