/*
 * Created on jul 16, 2005
 *
 * Copyright 2004 IDCA. All rights reserved.
 * IDCA PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package	com.idca.fsim.core.plugins;

import	com.idca.fsim.core.FsimContext;
import	com.idca.fsim.models.Word;
import	com.idca.fsim.models.Hit;

import	java.util.Date;
import	java.util.logging.Logger;

import	java.sql.Connection;
import	java.sql.ResultSet;
import	java.sql.SQLException;
import	java.sql.Statement;

/**
 * An DataBase indexer
 * 
 * @author Willem Cazander
 * @version 1.0 
 */
public class DBIndexer implements Indexer
{
	/** */
	private FsimContext fc = null;
	/** */
	private Logger logger = Logger.getLogger(DBIndexer.class.getName());
	
	private long queryLimit = 50000;
	
	private long queryOffset = 0;
	
	/**
	 * 
	 * @param fc
	 */
	public void setFsimContext(FsimContext fc)
	{
		this.fc=fc;
	}
	
	public void index(SearchSource searchSource)
	{
	    DBSearchSource source = (DBSearchSource)searchSource;
	    
	    
	    
		try
		{
			logger.info("start indexing... at startIndex="+queryOffset);
			Connection connection = fc.getConnectionByName(source.getDBName());
			Statement stmt = connection.createStatement();
			
			
			String sql = "select * from "+source.getTableName()+" where "+source.getIDColumnName()+" >= "+queryOffset+" order by "+source.getIDColumnName()+" ASC  limit "+queryLimit;
			logger.info("SQL:"+sql);
			ResultSet rs = stmt.executeQuery(sql);
			source.setLastIndexed(new Date());
			int sleep = 0;			
			long records = 0;
			
			while(rs.next())
			{
				records++;
				int id = rs.getInt(source.getIDColumnName());
				String columnName = source.getSearchColumnName();
				
				String text = rs.getString(columnName);
				String words[] = text.split(" ");
				for(int i=0;i<words.length;i++)	{
					index(id,words[i].toLowerCase(),source);
				}
				
				sleep++;
				if(sleep>100)
				{
				/*
					try {
						Thread.sleep(100);
					} catch (Exception e) {
						e.printStackTrace();
					}
					*/
					sleep=0;
				
					if(connection.isClosed())
					{
						logger.info("Lost connection going futher at index: "+(id+1));
						// maybe somethimes happens. 
						//index(source,id+1);
						break;
					}
				}
			}
			rs.close();
			if(stmt!=null)
			{
				// after slepp..
				stmt.close();
			}
			connection.close();
			
			
			if (records<queryLimit) {
				logger.info("Resteting index to begin.");
				queryOffset = 0; // reset to begining
			} else {
				queryOffset += queryLimit;
			}
			
			logger.info("done indexing...");
		}
		catch (SQLException sx)
		{
			logger.warning("SQL error in Words: "+sx.getMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
			logger.warning("Error in Indexer: "+e.getMessage());
		}
	}
	
	private void index(int id,String text,SearchSource source)
	{
		Word word = source.getWordLists().getWord(text);
		if(word==null)
		{
			word = new Word();
			word.text = text;
			source.getWordLists().put(word);
			
			Hit hit = new Hit();
			hit.id = id;
			hit.source = source;
			synchronized(word.hits)
			{
				word.hits.add(hit);
			}
			synchronized(source)
			{
				source.addHit(hit);
			}
		}
	}
}
