/*
 * 
 * 
 */

package	com.idca.fsim.searchers.files;

import com.idca.fsim.core.FsimContext;
import	com.idca.fsim.searchers.files.models.*;


import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import	java.util.Collection;
import	java.util.ArrayList;
import	java.util.List;
import	java.util.Iterator;
import	java.util.Hashtable;
import	java.util.HashSet;
import	java.util.logging.Logger;
import	java.util.Map;
import	java.util.List;

/**
 * Caches the paths.
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 22, 2006
 */
public class PathManager
{
	private Map paths = new Hashtable(2000);
	private Map hosts = new Hashtable(50);
	private Logger logger = Logger.getLogger(PathManager.class.getName());
	private FsimContext fsimContext = null;
	private String dbName = null;
	
	public PathManager(FsimContext fsimContext,String dbName)
	{
		this.fsimContext=fsimContext;
		this.dbName=dbName;
	}
	
	
	// ===== update cache.
	
	public void updatePaths()
	{
		try
		{
			logger.info("Building path cache of db: "+dbName);
			Connection connection = fsimContext.getConnectionByName(dbName);
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(
					"SELECT path.id,path.directory,path.parent_id,host.id as host_id,host.hostname,spider.udate"+
					" FROM path"+
					" LEFT JOIN host ON path.host_id = host.id"+
					" LEFT JOIN spider ON host.last_spider_id = spider.id"+
					" WHERE path.id>="+paths.size()+" ORDER BY id ASC");
				
			while(rs.next())
			{
				long id = rs.getLong("id");

				if(paths.containsKey(new Long(id)))
				{
					continue;
				}
				
				Path path = new Path();
				path.setID(id);
				path.setPath(rs.getString("directory"));
				
				Host host = getHost(rs.getString("hostname"));
				if(host==null) {
					host = new Host();
					host.setID(rs.getLong("host_id"));
					host.setHostName(rs.getString("hostname"));
					host.setLastScanned(rs.getDate("udate"));
					host.setRootPaths(new HashSet(4));
					host.setProtocol("ftp://"); // rs.getString("url"));
					hosts.put(host.getHostName(),host);
				}
				path.setHost(host);
				
				//int parentID = rs.getInt("parent");
				if(rs.getObject("parent_id")==null) {
					host.getRootPaths().add(path);
				} else {
					Path parent = (Path)paths.get(new Long(rs.getLong("parent_id")));
					path.setParent(parent);
					parent.getChilderen().add(path);
				}
				path.setChilderen(new HashSet(4));
				paths.put(new Long(path.getID()),path);
			}
			logger.info("cached total of paths: "+paths.size());
			logger.info("cached total of hosts: "+hosts.size());
			
			rs.close();
			stmt.close();
			connection.close();
		}
		catch (Exception sx)
		{
			System.err.println("error in Paths: "+sx.getMessage());
			sx.printStackTrace();
		}
	}
	
	// ======= paths.
	
	public Path getPath(long pathID)
	{
		return (Path)paths.get(new Long(pathID));
	}
	
	// ======= path browsing.
	
	public Collection getPaths(Path path)
	{
		return path.getChilderen();
	}
	public List getFiles(Path path)
	{
		List result = new ArrayList(20);
		try 
		{
			Connection connection = fsimContext.getConnectionByName(dbName);
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(
					"SELECT file.name,file.size,file.insert_date "+
					" FROM file"+
					" WHERE file.path_id = "+path.getID()+" ORDER BY file.id ASC");
			
			while(rs.next())
			{
				File file = new File();
				file.setName(rs.getString("name"));
				file.setPath(path);
				file.setSize(rs.getInt("size"));
				result.add(file);
			}
			
			rs.close();
			stmt.close();
			connection.close();
		} catch (Exception sx) {
			System.err.println("error in Paths: "+sx.getMessage());
			sx.printStackTrace();
		}
		System.out.println("FILES "+result);
		return result;
	}
	
	/**
	 * note: the file objects have no path !!
	 * @param host
	 * @return
	 */
	public List getRootFiles(Host host)
	{
		List result = new ArrayList(20);
		try 
		{
			Connection connection = fsimContext.getConnectionByName(dbName);
			Statement stmt = connection.createStatement();
			ResultSet rs = stmt.executeQuery(
					"SELECT file.name,file.size,file.insert_date,path.id as path_id "+
					" FROM file"+
					" LEFT JOIN path ON file.path_id = path.id"+
					" LEFT JOIN host ON path.host_id = host.id"+
					" WHERE host.id ="+host.getID()+" AND path.parent_id = null ORDER BY file.id ASC");
			
			while(rs.next())
			{
				File file = new File();
				file.setName(rs.getString("name"));
				Path path =(Path)paths.get(new Long(rs.getLong("path_id")));
				file.setPath(path);
				file.setSize(rs.getInt("size"));
				result.add(file);
			}
		}
		catch (Exception sx)
		{
			System.err.println("error in Paths: "+sx.getMessage());
			sx.printStackTrace();
		}
		System.out.println("ROOT FILES "+result);
		return result;		
	}
	
	// ====== hosts
	
	public Collection getHosts()
	{
		return hosts.values();
	}
	public Host getHost(String name)
	{
		return (Host)hosts.get(name);
	}
	public Host getHost(long hostID)
	{
		Iterator i = hosts.values().iterator();
		while(i.hasNext())
		{
			Host host =(Host)i.next();
			if(host.getID()==hostID)
			{
				return host;
			}
		}
		return null;
	}
}
