/*
 * 
 */

package	com.idca.fsim.web.session;

import	com.idca.fsim.core.FsimContext;
import	com.idca.fsim.core.Searcher;
import	com.idca.fsim.core.plugins.ResultCollector;
import com.idca.fsim.core.plugins.SearchSource;
import	com.idca.fsim.models.Hit;
import	com.idca.fsim.models.ModelHitTuble;

import	java.util.Collection;
import	java.util.ArrayList;
import	java.util.Iterator;
import	java.util.List;
import	java.util.Hashtable;
import	java.util.Map;
import	java.util.HashMap;
import	java.util.Locale;

/**
 * Provides the logic backend for the
 * web searcher
 * 
 * @author Willem Cazander
 * @version 1.0 25/07/2005 
 */
public class SearchBean
{
	/** */
	private FsimContext fsimContext = null;
	// some user var.
	private Locale locale = null;
	private int resultsPageSize = 250;
	private int keywordsListSize = 25;
	
	// search data
	private String searchQuery = "";
	private String lastSearchQuery = "";
	private Map searchSources = new Hashtable(2);
	private String searchSources2 = "";
	private String searchSources3 = "";
	private List searchHits = new ArrayList(1);
	private Collection searchPageHits = null;
	private int pageIndex = 0;
	private int pagePageIndex = 0;
	private int maxPages = 13;
	private List modelHitTublePageResults = new ArrayList(1);
	
	public SearchBean(FsimContext fsimContext)
	{
		this.fsimContext=fsimContext;
	}
	
	
	// =================== user prefences
	
	public void setLocale(Locale locale)
	{
		this.locale = locale;
	}
	public Locale getLocale()
	{
		return locale;
	}
	
	public void setResultsPageSize(int resultsPageSize)
	{
		this.resultsPageSize = resultsPageSize;
	}
	public int getResultsPageSize()
	{
		return resultsPageSize;
	}
	
	public void setKeywordsListSize(int keywordsListSize)
	{
		this.keywordsListSize=keywordsListSize;
	}
	public int getKeywordsListSize()
	{
		return keywordsListSize;
	}
	
	// ================= search logic.
	
	public void setSearchSources(String ss)
	{
		searchSources2 = ss;
	}
	
	
	public void addSearchSource(SearchSource searchSource)
	{
		searchSources.put(searchSource.getName(),searchSource);
	}
	public void removeSearchSource(String searchSourceName)
	{
		searchSources.remove(searchSourceName);
	}
	public Collection getSearchSources()
	{
		return searchSources.values();
	}
	
	//====
	
	public void setSearchQuery(String searchQuery)
	{
		if(searchQuery==null)
		{
			searchQuery="";
		}
		this.searchQuery=searchQuery;
	}
	public String getSearchQuery()
	{
		return searchQuery;
	}
	
	// ====
	
	public void setPageIndex(int pageIndex)
	{
		this.pageIndex = pageIndex*resultsPageSize;
		if(getHasMorePages())
		{
			pagePageIndex = pageIndex;
		}
		
	}
	public int getPageIndex()
	{
		return pageIndex;
	}
	
	
	// ========== The real search
	
	public void doSearch()
	{
		//fsimContext.getSearchSource("fsim-files").getWordLists().dumpInfo();
		
		if("".equals(searchQuery))
		{
			// clear results.
			searchHits = new ArrayList(1);
			lastSearchQuery = "";
			pageIndex = 0;
			modelHitTublePageResults = new ArrayList();
			return;
		}
		
		// reset search on change
		if (searchSources2.equals(searchSources3)==false) {
			searchSources3 = searchSources2;
			lastSearchQuery = "old";
		}
		
		
		// only search when new search query is given.
		if(!lastSearchQuery.equals(searchQuery))
		{
			
			searchHits= new ArrayList(400);
			for (Object ss1:fsimContext.getSearchSources()) {
				SearchSource ss = (SearchSource)ss1;
				if (searchSources2.contains(ss.getName())) {
					Searcher searcher = new Searcher(fsimContext.getSearchSource(ss.getName()).getWordLists());
					//searcher.setSearchSources(searchSources2);
					Collection hits = searcher.searchHits(searchQuery,true);
					searchHits.addAll(hits);
				}
			}
			

			lastSearchQuery=searchQuery;
		}
		
		// build pageHits
		if(pageIndex>searchHits.size())
		{
			pageIndex=searchHits.size();
			pagePageIndex=(searchHits.size()/resultsPageSize)-maxPages;
			if(pagePageIndex<0) {
				pagePageIndex = 0;
			}
		}
		if(pageIndex<0)
		{
			pageIndex=0;
			pagePageIndex=0;
		}
		int toIndex = pageIndex+resultsPageSize;
		if(toIndex>searchHits.size())
		{
			toIndex=searchHits.size();
		}
		searchPageHits =  searchHits.subList(pageIndex,toIndex);
					
		// build l
		modelHitTublePageResults = new ArrayList(resultsPageSize);
		if(searchPageHits.size()==0)
		{
			// nothing to get.
			return;
		}
		
		// split page results into different lists of same source.name.
		HashMap collectors = new HashMap(2);
		Iterator i = searchPageHits.iterator();
		while(i.hasNext())
		{
			Hit hit = (Hit)i.next();
			List list = (List)collectors.get(hit.source);
			if(list==null)
			{
				list = new ArrayList();
				collectors.put(hit.source,list);
			}
			list.add(hit);
		}
		// run all collectors
		i = collectors.keySet().iterator();
		while(i.hasNext())
		{
			SearchSource ss = (SearchSource)i.next();
			List hits = (List)collectors.get(ss);
			modelHitTublePageResults = ss.getResultCollector().getResults(hits,fsimContext,modelHitTublePageResults);
		}
		
		// last sort to the searchPageHits.
		List sorted = new ArrayList(resultsPageSize);
		i = searchPageHits.iterator();
		while(i.hasNext())
		{
			Hit hit = (Hit)i.next();
			//TODO: add some speed improment here
			// (its only for page 10-100 page result on the screen.)
			Iterator t = modelHitTublePageResults.iterator();
			while(t.hasNext())
			{
				ModelHitTuble modelHit = (ModelHitTuble)t.next();
				if(modelHit.getHit().equals(hit))
				{
					sorted.add(modelHit);
					break;
				}
			}	
		}
		modelHitTublePageResults = sorted;
	}
	
	// ======== Get the search results.
	
	public Collection getModelHitTublePageResults()
	{
		return modelHitTublePageResults;
	}
	

	public List getSearchHits()
	{
		return searchHits;
	}
	

	/**
	 * geeft alleen de viewable pages weer.
	 * @return
	 */
	public List getPages()
	{
		// calc number of pages
		ArrayList pages = new ArrayList(2);
		int p = searchHits.size()/resultsPageSize;
		p++;
		//System.out.println("pages p="+p+" maxp="+maxPages+" ppi="+pagePageIndex);
		for(int page=pagePageIndex;page<(maxPages+pagePageIndex) & p>page && p>1;page++) {
			//System.out.println("page="+page+" < "+(maxPages+pagePageIndex));
			pages.add(new Integer(page));
		}
		return pages;
	}
	public boolean getHasMorePages()
	{
		//System.out.println("sh="+searchHits.size()+"rps="+resultsPageSize);
		int p = searchHits.size()/resultsPageSize;
		p++;
		//System.out.println("has more p="+p+" maxp="+maxPages+" ppi="+pagePageIndex);
		if(p-maxPages>pagePageIndex)
		{
			//System.out.println("has more pages -- true");
			return true;
		}
		return false;
	}
}
